@@ -16,6 +16,15 @@ class ProblemNotFoundError(Exception):
         super().__init__(*args)
 
 
+class TemplateNotFoundError(Exception):
+    """
+    TemplateNotFoundError
+    """
+
+    def __init__(self, *args: object) -> None:
+        super().__init__(*args)
+
+
 class ProblemSet:
     """
     A set of problems, usually load from problem.yaml
@@ -27,6 +36,63 @@ class ProblemSet:
             self.dat = yaml.safe_load(f)
         self.console = console
 
+    def get_by_contest_problem_id(self, contest_id: str, problem_id: str) -> dict:
+        """
+        Get a problem by contest_id and problem_id
+        raise ProblemNotFoundError if not found
+        """
+        for problem in self.dat["problems"]:
+            if (
+                problem["contest_id"] == contest_id
+                and problem["problem_id"] == problem_id
+            ):
+                return problem
+        raise ProblemNotFoundError
+
+    def get_default_template(self, contest_id: str, problem_id: str) -> dict:
+        """
+        Get a problem's default template.
+        try f"{contest_id}_{problem_id}" first, then random file.
+        """
+        problem = self.get_by_contest_problem_id(contest_id, problem_id)
+        # detect {contest_id}_{problem_id} first
+        for template in problem["templates"]:
+            template_path = pathlib.Path(template["path"])
+            name = template_path.name.replace(f".{template['template']}", "")
+            if name == f"{contest_id}_{problem_id}":
+                return template
+        if len(problem["templates"]) == 0:
+            raise TemplateNotFoundError
+        return problem["templates"][0]
+
+    def get_by_contest_problem_id_file(
+        self, contest_id: str, problem_id: str, path: pathlib.Path
+    ) -> dict:
+        """
+        Get a template by contest_id, problem_id and path.
+        Argument "path" can be any of these:
+            1. A full path, like "~/abc123/abc123_a/abc123_a.cpp"
+            2. A file name, like "abc123_a.cpp"
+            3. A file name without extension, like "abc123_a"
+                WARNING: if "abc123_a.cpp" and "abc123_a.py" under this dir, idk what will be used.
+        """
+        path_string = pathlib.Path(path)
+        path_string_name = path_string.name
+        problem = self.get_by_contest_problem_id(contest_id, problem_id)
+        for template in problem["templates"]:
+            template_path_name = pathlib.Path(template["path"]).name
+            template_path_name_without_ext = template_path_name.replace(
+                f".{template['template']}", ""
+            )
+            if "." in path_string_name and template_path_name == path_string_name:
+                return template
+            if (
+                "." not in path_string_name
+                and template_path_name_without_ext == path_string_name
+            ):
+                return template
+        raise TemplateNotFoundError
+
     def add_problem(self, contest_id: str, problem_id: str) -> None:
         """
         Add a problem to this problem object.
@@ -133,7 +199,7 @@ def load_problem_from_all_ancestors(path_str: str, console: Console) -> ProblemS
         path = path.parent
 
 
-def get_problem_name(path_str: str, problems: ProblemSet, console: Console):
+def get_problem_name(path_str: pathlib.Path, problems: ProblemSet, console: Console):
     """
     Get Problem by path and problem object from parent
     like: getProblemName(
